###
### Plot the result of the function
###
### date: 1 January, 1997
###       14 March, 2000 - added title.use, xlab, ylab options
###       19 Dec, 2000 - added in lty option for plot0.roc
###
### sample call:  plot(roc(y~mark1+mark2,data=d1))
###               or
###               fit <- roc(status~mark1+mark2,data=d1)
###               plot(fit,type=2,ask=T)

plot.roc <- function(roc.obj, type=1, x.orient=1, ask=F, legend.use=T,
		     cost.fn=.5, cost.fp=.5, prevalence=c(.25,.5,.75),
		     xlab='', ylab='',title.use=T,lty=1:100)

{

### Taken from Trevor Hastie - s-news list ###
### 5/25/93                                ###
### Joins lists together as character      ###

joinup <- function(..., sep = " ", collapse = NULL)
{
        dot.list <- list(...)
        dot.list <- lapply(dot.list, as.character)
        do.call("paste", c(dot.list, list(sep = sep, collapse = collapse)))
}


## Plot the roc curve: sensitivity vs specificity (or 1-specificity) 
## Assume information here is for more than 1 marker

plot0.roc <- function(x.orient, roc.obj,legend.use, xlab, ylab, lty)
        {

         nmark <- ncol(roc.obj$x)
         sens <- roc.obj$sensitivity
         spec <- roc.obj$specificity
         marker.names <- attr(roc.obj$terms,"term.labels")
         x <- roc.obj$x
	 y <- roc.obj$y
	 lty.col <- lty

         for(k in 1:nmark) {
             o.par <- par(pty="s")
 	     on.exit(par(o.par))
             ord.m <- order(x[,k])
             ordx <- x[ord.m,k]
             ordy <- y[ord.m]
             ordsens <- sens[ord.m,k]
             ordspec <- spec[ord.m,k]
             use.ok <- match(unique(ordx),ordx)
 
         if(x.orient==1)
           { 
	    xlab2 <- ifelse(xlab=='','1-Specificity',xlab)
	    ylab2 <- ifelse(ylab=='','Sensitivity',ylab)
            plot(c(0,1),c(0,1),type="n", xlab=xlab2,ylab=ylab2)
            lines(1-ordspec[use.ok], ordsens[use.ok], type="s",
                  lty=lty.col, col=lty.col)
            abline(0,1,lty=1,col=1)
            par(new=T)
	    }
         if(x.orient==2)
           { 
     	     xlab2 <- ifelse(xlab=='','Specificity',xlab)
	     ylab2 <- ifelse(ylab=='','Sensitivity',ylab)

             plot(c(0,1),c(0,1),type="n",xlab=xlab2,ylab=ylab2)
             lines(ordspec[use.ok], ordsens[use.ok], type="s", lty=lty.col, 
		  col=lty.col)
             abline(1,-1,lty=1, col=1)
             par(new=T)
	     }
	   }
     if(legend.use) legend(locator(1),legend=marker.names,lty=1+(1:nmark), 
		       col=1+(1:nmark),cex=.8)

       }
## Plot the roc curve: sensitivity vs specificity (or 1-specificity) 
## Assume information here is for 1 marker

plot1.roc <- function(x.orient, sensitivity, specificity, markername,
	              xlab, ylab, title.use)
        {
	  o.par <- par(pty="s")
	  on.exit(par(o.par))

	  if(x.orient==1) 
           { 
     	     xlab2 <- ifelse(xlab=='','1-Specificity',xlab)
	     ylab2 <- ifelse(ylab=='','Sensitivity',ylab)
	     
	     plot(c(0,1),c(0,1),type="n", xlab=xlab2,ylab=ylab2)
             lines(1-specificity, sensitivity, type="s",
                   lty=1, col=1)
             abline(0,1,lty=1,col=1)
           }
         if(x.orient==2)
          { 
     	     xlab2 <- ifelse(xlab=='','Specificity',xlab)
	     ylab2 <- ifelse(ylab=='','Sensitivity',ylab)

	     plot(c(0,1),c(0,1),type="n", xlab=xlab2,ylab=ylab2)
             lines(specificity, sensitivity, type="s", lty=1, 
		   col=1)
            abline(1,-1,lty=1, col=1)
          }
         if(title.use) title(paste("\nMarker =",markername),cex=.8)
	}

## Plot the marker vs sensitivity and the marker vs specificity 
## Assume information here is for 1 marker

plot2.roc <- function(x,sensitivity,specificity,markername,legend.use,xlab,ylab)
	  {

	    xlim <- range(x)
    	    xlab2 <- ifelse(xlab=='',markername,xlab)
	    ylab2 <- ifelse(ylab=='',' ',ylab)
	    
	    plot(c(xlim[1],xlim[2]),c(0,1),type="n",xlab=xlab2,ylab=ylab2,xlim=xlim)
            lines(x,sensitivity,lty=1,col=1)
            lines(x,specificity,lty=2,col=2)
	    axis(4)

            if(legend.use) legend(locator(1),
			 legend=c("Sensitivity","Specificity"), 
		         lty=1:2,col=1:2, bty="n",cex=.8)
	  }

## Plot a 2-sided histogram - for the marker = 0 and = 1
## Assume information here is for 1 marker

plot3.roc <- function(x, y, yname, markername, xlab, ylab)
        { 
          tmpx <- x
	  nclass <- log(length(tmpx),base=2)+1
          breaks <- pretty(tmpx,nclass)
	  if(any(tmpx<=breaks[1]))
             breaks <- c(breaks[1] - diff(breaks)[1],breaks)
          tmpx[tmpx> max(breaks)] <- max(breaks)
          bin0 <- cut(tmpx[y==0],breaks)
	  bin1 <- cut(tmpx[y==1],breaks)
	  counts0 <- tabulate(bin0,length(levels(bin0)))
	  counts1 <- tabulate(bin1,length(levels(bin1)))
	  xlim <- c(-max(c(counts0,counts1)),max(c(counts0,counts1)))
	  tmp <-barplot(counts1,horiz=T,xlim=xlim,axes=F)
	  par(new=T)
	  barplot(-counts0,horiz=T,axes=F, density=10,xlim=xlim)
	  axis(2,tmp,labels=attr(bin0,"levels"),las=1,cex=.8,ticks=F,col=1)
	  axis(1,col=1)
	  mtext(outer=F,paste(yname,"= 0              ",yname,"= 1"),
		side=3,line=0,col=1)

  	  xlab2 <- ifelse(xlab=='',markername,xlab)
	  ylab2 <- ifelse(ylab=='',' ',ylab)
          title(xlab=xlab2, ylab=ylab2)
	}

			  
## Plot the expected cost of mistakes (for a given set of prevalences) vs x
## Assume information here is for 1 marker

plot4.roc <- function(sensitivity, specificity, prevalence, 
		      cost.fp, cost.fn, x, markername,legend.use,xlab, ylab)
  
  { emc <- cost.fp*outer(1-specificity,1-prevalence) + 
           cost.fn*outer(1-sensitivity,prevalence)

    n.prev <- length(prevalence)

    xlim <- range(x,na.rm=T)
    ylim <- range(emc,na.rm=T)

    xlab2 <- ifelse(xlab=='',' ',xlab)
    ylab2 <- ifelse(ylab=='','Expected cost of Mistakes',ylab)

    if(n.prev==1) plot(x,emc,xlim=xlim, ylim=ylim, xlab=xlab2,
  	                     ylab=ylab2,type="l")
    if(n.prev!=1){
       for(k in 1:n.prev) {

          lty.col <- k
          xlab2 <- ifelse(xlab=='',markername,xlab)
	  if(k==1)  plot(x,emc[,k],xlim=xlim,ylim=ylim,lty=lty.col,col=lty.col,
	            xlab=xlab2, ylab=ylab2,type="l")
          if(k>1) lines(x,emc[,k],lty=lty.col,col=lty.col)
	}
       if(legend.use) legend(locator(1),legend=c("Prev",as.character(prevalence)), 
	      lty=c(-4,1:n.prev), col=c(-4,1:n.prev),cex=.8)
     }
  }
	  
## Plot the information content (for a given set of prevalences) vs x
## Assume information here is for 1 marker

plot5.roc <- function(sensitivity, specificity, prevalence, x, 
		      markername=markername,legend.use,xlab,ylab)
  
  { part1 <- sensitivity/(outer(sensitivity,prevalence) +
			  outer(1-specificity,1-prevalence))
    part2 <- (1-specificity)/(outer(sensitivity,prevalence) +
			      outer(1-specificity,1-prevalence))
    part3 <- (1-sensitivity)/(1-outer(sensitivity,prevalence) - 
			      outer(1-specificity,1-prevalence))
    part4 <- specificity/(1-outer(sensitivity,prevalence) -
			  outer(1-specificity,1-prevalence))

    ic <- outer(sensitivity,prevalence)*ifelse(part1<=0,0,log(part1,2)) +
          outer(1-specificity,1-prevalence)*ifelse(part2<=0,0,log(part2,2)) +
	  outer(1-sensitivity,prevalence)*ifelse(part3<=0,0,log(part3,2)) +
          outer(specificity,1-prevalence)*ifelse(part4<=0,0,log(part4,2))  

    n.prev <- length(prevalence)

    xlim <- range(x,na.rm=T)
    ylim <- range(ic,na.rm=T)

    xlab2 <- ifelse(xlab=='',' ',xlab)
    ylab2 <- ifelse(ylab=='','Information content',ylab)

    if(n.prev==1) plot(x,ic,xlim=xlim, ylim=ylim, xlab=xlab2, ylab=ylab2,type="l")
    if(n.prev!=1){
       for(k in 1:n.prev) {

          lty.col <- k
          xlab2 <- ifelse(xlab=='',markername,xlab)
          if(k==1) plot(x,ic[,k],xlim=xlim,ylim=ylim,lty=lty.col,col=lty.col,
	           xlab=xlab2, ylab=ylab2,type="l")
	  if(k>1) lines(x,ic[,k],lty=lty.col,col=lty.col)
          }
       if(legend.use) legend(locator(1),legend=c("Prev",as.character(prevalence)), 
	      lty=c(-4,1:n.prev), col=c(-4,1:n.prev),cex=.8)
       }
  }
	  

  nmark <- ncol(roc.obj$x)
  sens <- roc.obj$sensitivity
  spec <- roc.obj$specificity
  marker.names <- attr(roc.obj$terms,"term.labels")

  if(!ask) {
      if(type==0) plot0.roc(x.orient=x.orient,roc.obj=roc.obj,
			    legend.use=legend.use,xlab=xlab,ylab=ylab,lty=lty) 
      if(type>0) {	
      for(k in 1:nmark) {

      ord.m <- order(roc.obj$x[,k])
      ordx <- roc.obj$x[ord.m,k]
      ordy <- roc.obj$y[ord.m]
      ordsens <- sens[ord.m,k]
      ordspec <- spec[ord.m,k]
      use.ok <- match(unique(ordx),ordx)
      yname <- attr(roc.obj$terms,"variables")[[1]]
      markername <- marker.names[k]

      if(type==1) plot1.roc(x.orient=x.orient,
	                    sensitivity=ordsens,specificity=ordspec, 
	                    markername=markername,title.use=title.use,
			    xlab=xlab,ylab=ylab)
      if(type==2) plot2.roc(x=ordx,sensitivity=ordsens,specificity=ordspec, 
	                    markername=markername,legend.use=legend.use,
			    xlab=xlab,ylab=ylab)
      if(type==3)
      plot3.roc(x=ordx,y=ordy,yname=yname,markername=markername,
	        xlab=xlab,ylab=ylab)
      if(type==4) plot4.roc(sensitivity=ordsens,specificity=ordspec,
	                    prevalence=prevalence, cost.fp=cost.fp,
	                    legend.use=legend.use, ylab=ylab,xlab=xlab,
	                    cost.fn=cost.fn, x=ordx, markername=markername)
      if(type==5) plot5.roc(sensitivity=ordsens,specificity=ordspec,
	                    prevalence=prevalence, x=ordx,xlab=xlab,ylab=ylab,
                            markername=markername,legend.use=legend.use)
      }
     }
    }

  if(ask) {

    typemenu <- paste("plot type (", type,")",sep="")
    orientmenu <- paste("x orientation (",x.orient,")",sep="")
    prevmenu <- paste("prevalences (",joinup(prevalence,collapse=","),
		      ")",sep="")
    fnmenu <- paste("cost of false negative (",cost.fn,")",sep="")
    fpmenu <- paste("cost of false positive (",cost.fp,")",sep="")
    browsemenu <- "browser() - add extras to plots using this option"

    tmenu <- c(paste("plot marker",marker.names),
	       "plot all roc curves on one plot", typemenu, 
               orientmenu, prevmenu, fnmenu, 
	       fpmenu,browsemenu)
    pick <- 1

    while(pick > 0 && pick <= length(tmenu)){
          pick <- menu(tmenu, title=
                       "\nMake a plot selection (or 0 to exit):\n")
          if(pick > 0 && pick <= length(marker.names)) {

            ord.m <- order(roc.obj$x[,pick])
	    ordx <- roc.obj$x[ord.m,pick]
	    ordy <- roc.obj$y[ord.m]
	    ordsens <- sens[ord.m,pick]
	    ordspec <- spec[ord.m,pick]
	    use.ok <- match(unique(ordx),ordx)
	    yname <- attr(roc.obj$terms,"variables")[[1]]
	    markername <- marker.names[pick]

	    if(type==1) plot1.roc(x.orient=x.orient,
	                    sensitivity=ordsens,specificity=ordspec, 
	                    markername=markername,xlab=xlab,ylab=ylab,
			    title.use=title.use)
	    if(type==2) plot2.roc(x=ordx,sensitivity=ordsens,legend.use=legend.use,
		            specificity=ordspec,
			    markername=markername,
			    xlab=xlab,ylab=ylab)
	    if(type==3) plot3.roc(x=ordx,y=ordy,xlab=xlab,ylab=ylab,
                            yname=yname,markername=markername)
	    if(type==4) plot4.roc(sensitivity=ordsens,specificity=ordspec,
	                    prevalence=prevalence, cost.fp=cost.fp,
                            legend.use=legend.use,xlab=xlab,ylab=ylab,
	                    cost.fn=cost.fn, x=ordx, markername=markername)
	    if(type==5) plot5.roc(sensitivity=ordsens,specificity=ordspec,
	                    prevalence=prevalence,legend.use=legend.use, 
		            x=ordx,markername=markername,xlab=xlab,ylab=ylab)
	  }
	          
    else switch(pick - length(marker.names),
	 {
	 plot0.roc(x.orient=x.orient,roc.obj=roc.obj,legend.use=legend.use,
	           xlab=xlab,ylab=ylab,lty=lty) },
         { cat("\n1:roc curve\n2:sens vs spec\n3:histogram\n4:cost mistake by prevalences\n5:information content by prevalences\ntype in new plot number here")
	   type <- eval(parse(), local=F)
      	   typemenu <- paste("plot type (", type,")",sep="")
         },
         { cat("!!Affects plot type 1 only!!\n1: 1-Specificity\n2: Specificity\ntype in x orientation(1-2) here")
	   x.orient <- eval(parse(), local=F)
	   orientmenu <- paste("x orientation (",x.orient,")",sep="")
	 },
	 { cat("type in a list of prevalences (between 0 and 1) c( ):\n")
	   prevalence <- eval(parse(),local=F)
	   prevmenu <- paste("prevalences (",joinup(prevalence,collapse=","),
		      ")",sep="")
	 },
	 { cat("type in penalty for false negatives (between 0 and 1)\n")
	   cost.fn <- eval(parse(), local=F)
           fnmenu <- paste("cost of false negative (",cost.fn,")",sep="")
	 },
	 { cat("type in penalty for false positives (between 0 and 1)\n")
	   cost.fp <- eval(parse(), local=F)
           fpmenu <- paste("cost of false positive (",cost.fp,")",sep="")
	 },
         { cat("\nBrowser is on (0 to exit)\n")
	   browser(x)
         },
	 invisible(return(x)))

         tmenu <- c(paste("plot marker",marker.names),
	           "plot all roc curves on one plot", typemenu, 
                    orientmenu, prevmenu, fnmenu, 
	            fpmenu,browsemenu)

         }  
    invisible(x)
  }
}

